// hooks
import { useTranscriptPopover } from '../hooks/useTranscriptPopover';
// @mui
import { styled } from '@mui/material/styles';
import { Typography } from '@mui/material';
// components
import DetailedTranscriptTooltip from './DetailedTranscriptTooltip';
import { useMemo } from 'react';
import { compareTexts } from 'src/utils/text';

const Root = styled('div')(() => ({
  width: '100%',
  paddingBlock: '5px',
}));

const Marking = styled('span')(() => ({
  paddingInline: '3px',
  cursor: 'pointer',
}));

const Connectives = styled(Marking)(({ theme }) => ({
  backgroundColor: theme.palette.chart.violet[3],
}));

const Filler = styled(Marking)(({ theme }) => ({
  backgroundColor: theme.palette.chart.yellow[3],
}));

const Pause = styled(Marking)(({ theme }) => ({
  backgroundColor: theme.palette.chart.red[3],
}));

const Repetition = styled(Marking)(({ theme }) => ({
  backgroundColor: theme.palette.chart.blue[3],
}));

type DetailedTranscriptProps = {
  transcript: string;
  speechPhrase?: string;
};

export default function DetailedTranscript(props: DetailedTranscriptProps) {
  const { transcript, speechPhrase = '' } = props;
  const cleanedTranscript = transcript
    .trim()
    .replace(/[.,!\?;]$/g, '')
    .replace(/<[^>]*>/g, '');
  const mistakes = useMemo(
    () => compareTexts(speechPhrase, cleanedTranscript),
    [cleanedTranscript, speechPhrase]
  );
  const { anchorEl, open, title, description, handlePopoverOpen, handlePopoverClose } =
    useTranscriptPopover();

  const parseTranscript = (transcript: string) => {
    transcript = '<text>' + transcript + '</text>';
    transcript = transcript.replace(/&/g, '&amp;');
    const parser = new DOMParser();
    const xml = parser.parseFromString(transcript, 'text/xml');
    if (!xml.getElementsByTagName('text').item(0)?.childNodes) return <></>;

    const nodes = xml.getElementsByTagName('text').item(0)?.childNodes as NodeListOf<ChildNode>;
    const elements = [];
    let wordIdx = 0;
    for (let idx = 0; idx < nodes.length; idx++) {
      const node = nodes[idx];
      if (node.nodeName === 'discourse-marker' && !mistakes[wordIdx]?.correct) {
        elements.push(
          <Connectives
            key={`transcript-${idx}`}
            onMouseEnter={(event) =>
              handlePopoverOpen(
                event,
                'Connective',
                `Used for: ${(node as HTMLElement).getAttribute('description')}`
              )
            }
            onMouseLeave={handlePopoverClose}
          >
            {node.textContent}
          </Connectives>
        );
      } else if (node.nodeName === 'filler-word') {
        elements.push(
          <Filler
            key={`transcript-${idx}`}
            onMouseEnter={(event) => handlePopoverOpen(event, 'Filler word', '')}
            onMouseLeave={handlePopoverClose}
          >
            {node.textContent}
          </Filler>
        );
      } else if (node.nodeName === 'word-repetition') {
        elements.push(
          <Repetition
            key={`transcript-${idx}`}
            onMouseEnter={(event) => handlePopoverOpen(event, 'Word repetition', '')}
            onMouseLeave={handlePopoverClose}
          >
            {node.textContent}
          </Repetition>
        );
      } else if (node.nodeName === 'speech-pause') {
        elements.push(
          <Pause
            key={`transcript-${idx}`}
            onMouseEnter={(event) =>
              handlePopoverOpen(
                event,
                'Pause',
                `Duration: ${(node as HTMLElement).getAttribute('duration_seconds')} seconds`
              )
            }
            onMouseLeave={handlePopoverClose}
          >
            {'[ pause ]'}
          </Pause>
        );
      } else {
        elements.push(node.textContent);
      }

      wordIdx += node.textContent?.trim()?.split(/\s/).length ?? 0;
    }
    return <>{elements}</>;
  };

  return (
    <Root>
      <Typography>{parseTranscript(transcript)}</Typography>
      <DetailedTranscriptTooltip
        open={open}
        anchorEl={anchorEl}
        title={title}
        description={description}
        handlePopoverClose={handlePopoverClose}
      />
    </Root>
  );
}
