// @mui
import { styled } from '@mui/material/styles';
import {
  Chip,
  Card,
  CardProps,
  Typography,
  Button,
  Stack,
  StackProps,
  ButtonProps,
} from '@mui/material';
// components
import Image from './Image';

interface CardStyleProps extends CardProps {
  horizontal?: boolean;
  background?: string;
}
const CardStyle = styled(Card, {
  shouldForwardProp: (prop) => prop !== 'horizontal',
})<CardStyleProps>(({ theme, horizontal, background }) => ({
  cursor: 'pointer',
  display: 'flex',
  flexDirection: 'column',
  alignItems: 'stretch',
  padding: '20px',
  gap: '1rem',
  background: background ? background : theme.palette.background.paper,
  ...(horizontal && {
    flexDirection: 'row',
    gap: '16px',
    width: '100%',
    '& > *:first-of-type': {
      flexBasis: 'max(30%, 300px)',
    },
  }),
}));

interface ContainerStyleProps extends StackProps {
  horizontal?: boolean;
}
const ContainerStyle = styled(Stack, {
  shouldForwardProp: (prop) => prop !== 'horizontal',
})<ContainerStyleProps>(({ horizontal }) => ({
  display: 'flex',
  flexBasis: '100%',
  justifyContent: 'space-between',
  flexGrow: '1',
  gap: '30px',
  ...(horizontal && {
    flexDirection: 'row',
    alignItems: 'center',
    flexWrap: 'wrap',
    gap: '1rem',
  }),
}));

const ImageStyle = styled(Image)(() => ({
  width: '100%',
  height: '100%',
  objectFit: 'cover',
  borderRadius: '5px',
}));

const TitleStyle = styled(Typography)(({ theme }) => ({
  fontSize: theme.typography.h4.fontSize,
  fontWeight: theme.typography.h4.fontWeight,
  lineHeight: '30px',
  marginBottom: '10px',
}));

const DescriptionStyle = styled(Typography)(({ theme }) => ({
  fontSize: theme.typography.subtitle1.fontSize,
  lineHeight: '24px',
}));

const ChipStyle = styled(Chip)(({ theme }) => ({
  color: theme.palette.text.secondary,
  fontWeight: 700,
  lineHeight: '1rem',
  borderRadius: '0.25rem',
}));

interface ButtonStyleProps extends ButtonProps {
  horizontal?: boolean;
  buttonTextColor?: string;
  buttonBackground?: string;
}
const ButtonStyle = styled(Button, {
  shouldForwardProp: (prop) => prop !== 'horizontal',
})<ButtonStyleProps>(({ theme, horizontal, buttonTextColor, buttonBackground }) => ({
  ...theme.typography.subtitle1,
  width: 'fit-content',
  lineHeight: '24px',
  borderRadius: '4px',
  alignSelf: 'flex-end',
  padding: '8px 24px',
  '&:hover': {
    backgroundColor: buttonBackground ? buttonBackground : theme.palette.primary.main,
  },
  ...(horizontal && {
    alignSelf: 'unset',
    marginLeft: 'auto',
  }),
  color: buttonTextColor ? buttonTextColor : theme.palette.common.white,
  backgroundColor: buttonBackground ? buttonBackground : theme.palette.primary.main,
}));

type FeatureCardProps = {
  horizontal?: boolean;
  title: string;
  description: string;
  image: string;
  handleClick?: VoidFunction;
  buttonText?: string;
  aspectRatio?: string;
  tag?: string;
  background?: string;
  textColor?: string;
  buttonTextColor?: string;
  buttonBackground?: string;
};

export default function FeatureCard(props: FeatureCardProps) {
  const {
    horizontal,
    title,
    description,
    image,
    handleClick,
    buttonText,
    aspectRatio,
    tag,
    background,
    textColor,
    buttonTextColor,
    buttonBackground,
  } = props;

  return (
    <CardStyle horizontal={horizontal} raised={false} onClick={handleClick} background={background}>
      <div>
        <ImageStyle
          src={image}
          alt="Feature Image"
          ratio={aspectRatio ?? (horizontal ? '16/13.5' : '16/10')}
          customStyle={{ height: '100%' }}
        />
      </div>
      <ContainerStyle horizontal={horizontal}>
        <div>
          <Stack direction="row" justifyContent="space-between" alignItems="flex-start">
            <TitleStyle color={textColor ? textColor : 'inherit'}>{title}</TitleStyle>
            {tag && <ChipStyle label={tag} />}
          </Stack>
          <DescriptionStyle color={(theme) => (textColor ? textColor : theme.palette.grey[600])}>
            {description}
          </DescriptionStyle>
        </div>
        <ButtonStyle
          onClick={handleClick}
          variant="contained"
          disableElevation
          horizontal={horizontal}
          buttonTextColor={buttonTextColor}
          buttonBackground={buttonBackground}
        >
          {buttonText ?? 'Try it now'}
        </ButtonStyle>
      </ContainerStyle>
    </CardStyle>
  );
}
