import { isValidElement, useMemo } from 'react';
// @mui
import { alpha, styled } from '@mui/material/styles';
import { Card, Typography, Box, useTheme, Stack } from '@mui/material';
// components
import ScoreProgressIndicator from 'src/components/ScoreProgressIndicator';
import DetailedTranscriptTooltip from './DetailedTranscriptTooltip';
// hooks
import { useTranscriptPopover } from 'src/hooks/useTranscriptPopover';
// utils
import { compareTexts } from 'src/utils/text';
import { Feedback } from 'src/@types/feedback';

const CardStyle = styled(Card)(({ theme }) => ({
  width: '100%',
  display: 'flex',
  flexDirection: 'column',
  backgroundColor: alpha(theme.palette.primary.light, 0.05),
  boxShadow: `-2px 0px ${theme.palette.primary.light}`,
}));

const CardHeader = styled(Box)(() => ({
  display: 'flex',
  alignItems: 'center',
  justifyContent: 'space-between',
}));

const TitleStyle = styled(Typography)(({ theme }) => ({
  fontWeight: '700',
  fontSize: '16px',
  marginBlock: '8px',
  color: theme.palette.primary.main,
}));

const ValueStyle = styled(Typography)(({ theme }) => ({
  fontWeight: '700',
  fontSize: '16px',
  color: theme.palette.primary.main,
}));

const ContentStyle = styled(Typography)(({ theme }) => ({
  fontSize: '16px',
  color: theme.palette.text.primary,
  padding: '5px 0px',
}));

const BoldText = styled(Typography)(({ theme }) => ({
  color: theme.palette.text.primary,
  fontWeight: '700',
  fontSize: '16px',
}));

const TextScoreContainer = styled(Box)(({ theme }) => ({
  borderRadius: '100px',
  color: theme.palette.common.white,
  fontSize: '16px',
  fontWeight: '500',
  lineHeight: '20px',
  padding: '2px 8px',
}));

const ContentContainer = styled('div')(() => ({
  display: 'grid',
  gap: '1rem',
  gridTemplateColumns: 'repeat(auto-fit, minmax(350px, 1fr))',
}));

const Divider = styled('hr')(({ theme }) => ({
  borderBlockColor: theme.palette.divider,
  borderWidth: '0.5px',
}));

const Marking = styled('span')(() => ({
  paddingInline: '3px',
  cursor: 'pointer',
}));

export default function FeedbackCard(props: Feedback) {
  const { title, content, speechPhrase, value, children, plain } = props;
  const theme = useTheme();
  const stringScoreColor =
    title === 'Grammar complexity' || title === 'Vocabulary complexity'
      ? value === 'SIMPLE'
        ? theme.palette.chart.green[0]
        : value === 'AVERAGE'
          ? theme.palette.chart.yellow[0]
          : theme.palette.chart.red[0]
      : title === 'Content relevance score'
        ? value === 'RELEVANT'
          ? theme.palette.chart.green[0]
          : value === 'PARTIALLY RELEVANT'
            ? theme.palette.chart.yellow[0]
            : theme.palette.chart.red[0]
        : theme.palette.chart.green[0];

  const cleanedTranscript = content
    ? content
        .trim()
        .replace(/[.,!?;]$/g, '')
        .replace(/<[^>]*>/g, '')
    : '';

  const mistakes = useMemo(
    () => compareTexts(speechPhrase || '', cleanedTranscript),
    [cleanedTranscript, speechPhrase]
  );
  const {
    anchorEl,
    title: tooltipTitle,
    open,
    description,
    handlePopoverOpen,
    handlePopoverClose,
  } = useTranscriptPopover();

  return (
    <CardStyle>
      <Box style={{ padding: '10px 24px' }}>
        <CardHeader>
          <TitleStyle>{title}</TitleStyle>
          {title === 'Speech rate' ? (
            <ScoreProgressIndicator
              score={(value as number) || 0}
              min={0}
              max={280}
              firstMark={80}
              secondMark={200}
              captions={['Slow', 'Fast']}
            />
          ) : isValidElement(value) ? (
            value
          ) : plain ? (
            <ValueStyle>{value}</ValueStyle>
          ) : typeof value === 'string' ? (
            <TextScoreContainer
              sx={{
                backgroundColor: stringScoreColor,
              }}
            >
              {value.charAt(0).toUpperCase() + value.slice(1).toLowerCase()}
            </TextScoreContainer>
          ) : (
            <ValueStyle>{value}</ValueStyle>
          )}
        </CardHeader>
        {(content || children) && (
          <>
            <Divider />
            <BoldText>Feedback</BoldText>
            <ContentContainer>
              {speechPhrase ? (
                <Stack direction="row" flexWrap="wrap">
                  {mistakes.map(({ word, correct, substitutedWith }, idx) => (
                    <Marking
                      key={`unscripted-transcript-word-${idx}`}
                      onMouseEnter={(event) =>
                        !correct &&
                        handlePopoverOpen(
                          event,
                          substitutedWith ? 'Expected:' : 'Missing',
                          substitutedWith ?? ''
                        )
                      }
                      onMouseLeave={handlePopoverClose}
                      sx={(theme) => ({
                        color: correct
                          ? theme.palette.success.main
                          : substitutedWith
                            ? theme.palette.warning.main
                            : theme.palette.error.main,
                        textDecoration: correct || !substitutedWith ? 'none' : 'line-through',
                        cursor: correct ? 'unset' : 'pointer',
                      })}
                    >
                      {word}
                    </Marking>
                  ))}
                </Stack>
              ) : content ? (
                <ContentStyle>{content}</ContentStyle>
              ) : null}
              {children}
              <DetailedTranscriptTooltip
                open={open}
                anchorEl={anchorEl}
                title={tooltipTitle}
                description={description}
                handlePopoverClose={handlePopoverClose}
              />
            </ContentContainer>
          </>
        )}
      </Box>
    </CardStyle>
  );
}
