import { ChangeEvent, useState } from 'react';
import { useSnackbar } from 'notistack';
import useInterval from '../hooks/useInterval';
import useResponsive from '../hooks/useResponsive';
// @mui
import { styled } from '@mui/material/styles';
import { Typography, Stack, alpha, Button, TextField } from '@mui/material';
import { VolumeUp } from '@mui/icons-material';
// component
import RecordingSection from './RecordingSection';
import ReactPlayer from 'react-player';
import Image from '../components/Image';

const RootStyle = styled(Stack)(() => ({
  width: '100%',
  borderRadius: '10px',
  alignItems: 'center',
  textAlign: 'center',
  gap: '16px',
  '& > div:nth-child(2)': {
    marginTop: '30px',
  },
}));

const TitleStyle = styled(Typography)(({ theme }) => ({
  fontSize: theme.typography.h2.fontSize,
  fontWeight: theme.typography.h2.fontWeight,
  lineHeight: '48px',
  textAlign: 'center',
  marginTop: '20px',
}));

const TextFieldStyle = styled(TextField)(() => ({
  marginBlock: '20px 10px',
  width: '80%',
}));

const ButtonContainedStyle = styled(Button)(({ theme }) => ({
  padding: '12px 30px',
  fontSize: '16px',
  fontWeight: '500',
  lineHeight: '24px',
  color: theme.palette.common.black,
  borderRadius: '4px',
  marginBottom: '16px',
  backgroundColor: theme.palette.secondary.main,
  '&:hover': {
    backgroundColor: alpha(theme.palette.secondary.main, 0.7),
  },
}));

const ImageStyle = styled(Image)(() => ({
  borderRadius: '5px',
}));

const StackStyle = styled(Stack)(() => ({
  width: '100%',
  gap: '3rem',
  alignItems: 'center',
  justifyContent: 'center',
  '& > span': {
    flex: '1 1 100%',
    maxWidth: '600px',
  },
}));

type IeltsTestSectionProps = {
  text?: string;
  clearData: () => void;
  handleSubmitRecording: (blobFile: any) => void;
  handleUpdateDuration: (number: number) => void;
  blobUrl: string;
  customText: string;
  handleCustomTextChange: (event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => void;
  picture: string;
  videoUrl: string;
  minDuration: number;
  maxDuration: number;
};

export default function IeltsTestSection(props: IeltsTestSectionProps) {
  const {
    text,
    clearData,
    handleSubmitRecording,
    handleUpdateDuration,
    blobUrl,
    customText,
    handleCustomTextChange,
    picture,
    videoUrl,
    minDuration,
    maxDuration,
  } = props;
  const [recording, setRecording] = useState(false);
  const [playingRecording, setPlayingRecording] = useState(false);
  const [duration, setDuration] = useState(0);
  const { enqueueSnackbar } = useSnackbar();
  const isDesktop = useResponsive('up', 'md');

  useInterval(
    () => {
      setDuration(duration + 0.2);
      if (duration >= maxDuration) {
        setRecording(false);
      }
    },
    recording ? 200 : null
  );

  useInterval(
    () => {
      setPlayingRecording(false);
    },
    playingRecording ? (duration + 1) * 1000 : null
  );

  const handleRecording = () => {
    if (!recording) {
      setDuration(0);
      clearData();
    } else {
      if (duration < minDuration)
        enqueueSnackbar(`Please speak for at least ${minDuration} seconds.`, { variant: 'error' });
      else handleUpdateDuration(duration);
    }
    setRecording(!recording);
  };

  const handlePlayRecording = async () => {
    try {
      setPlayingRecording(true);
      const recordingAudio = new Audio(blobUrl);
      await recordingAudio.play();
    } catch (err) {
      console.log(err);
    }
  };

  return (
    <RootStyle>
      {!picture &&
        typeof text == 'string' &&
        (text !== '' ? (
          <TitleStyle>{text}</TitleStyle>
        ) : (
          <TextFieldStyle
            multiline
            placeholder="Type Your Question ...."
            value={customText}
            onChange={handleCustomTextChange}
          />
        ))}
      {(picture || videoUrl) && <div />}
      <StackStyle direction={isDesktop ? 'row' : 'column'}>
        {picture && <ImageStyle src={picture} alt="Picture to describe" />}
        {videoUrl && (
          <ReactPlayer
            style={{
              borderRadius: '5px',
              overflow: 'hidden',
              marginBottom: '1rem',
              maxWidth: '640px',
              aspectRatio: '16/9',
            }}
            url={videoUrl}
            controls
            width="80%"
            height="100%"
          />
        )}
        <div>
          <ButtonContainedStyle
            variant="contained"
            disableElevation
            startIcon={<VolumeUp />}
            disabled={playingRecording}
            onClick={handlePlayRecording}
            sx={{ marginBottom: '2rem' }}
          >
            Play Recording
          </ButtonContainedStyle>
          <RecordingSection
            recording={recording}
            duration={duration}
            maxDuration={maxDuration}
            preDescription={
              'You need to speak' +
              (minDuration ? ` more than ${minDuration} seconds but` : '') +
              ` no more than ${maxDuration + 1} seconds`
            }
            handleRecording={handleRecording}
            handleStop={handleSubmitRecording}
          />
        </div>
      </StackStyle>
    </RootStyle>
  );
}
