import { useMemo } from 'react';
//mui
import {
  Typography,
  Box,
  Stack,
  LinearProgress,
  linearProgressClasses,
  CircularProgress,
  circularProgressClasses,
  TypographyProps,
} from '@mui/material';
import { styled } from '@mui/material/styles';
// type
import { SkillResult } from '../@types/lcat';
// utils
import { CEFR_BANDS, scaleIeltsScore } from 'src/utils/scoreUtils';

const OverallContainer = styled(Box)(() => ({
  fontSize: 'clamp(40px, 14vw, 96px)',
  position: 'relative',
  alignSelf: 'center',
  height: 'fit-content',
}));

const ScoreStyle = styled(Typography)(() => ({
  position: 'absolute',
  inset: '0',
  margin: 'auto',
  width: 'fit-content',
  paddingTop: 'min(1em, 40px)',
  fontSize: 'clamp(30px, 10vw, 80px)',
  fontWeight: '700',
}));

const InnerCircularProgress = styled(CircularProgress)(() => ({
  position: 'absolute',
  left: 0,
  [`& .${circularProgressClasses.circle}`]: {
    strokeLinecap: 'round',
  },
}));

const BoldText = styled(Typography)(() => ({
  color: '#212B36',
  fontWeight: '700',
  fontSize: '16px',
  lineHeight: '1.65em',
}));

interface ScoreIndicatorTextProps extends TypographyProps {
  progressPosition: number;
}
const ScoreIndicatorText = styled(BoldText, {
  shouldForwardProp: (prop) => prop !== 'progressPosition',
})<ScoreIndicatorTextProps>(({ theme, progressPosition }) => ({
  color: theme.palette.primary.main,
  position: 'absolute',
  inset: `-35px auto auto calc(0.2em + (100% - 0.4em) * ${progressPosition}) !important`,
  transform: 'translate3d(-50%, 0, 0) !important',
}));

const TextStyle = styled(Typography)(() => ({
  fontSize: '16px',
  lineHeight: '1.65em',
}));

const ScoreMaxMinText = styled(TextStyle)(({ theme }) => ({
  fontWeight: '500',
  color: theme.palette.grey[600],
}));

const LinearProgressContainer = styled(Stack)(() => ({
  flexDirection: 'row',
  alignItems: 'center',
  justifyContent: 'space-between',
  width: '100%',
  gap: '0.5em',
  paddingTop: '1.8em',
  '& > span': {
    flex: ' 1 1 auto',
  },
}));

const LinearProgressBarContainer = styled('div')(() => ({
  position: 'relative',
  width: '100%',
}));

const BorderLinearProgress = styled(LinearProgress)(({ theme }) => ({
  height: 'clamp(12px, 3.3vw, 20px)',
  borderRadius: '100px',
  [`&.${linearProgressClasses.colorPrimary}`]: {
    backgroundColor: theme.palette.grey[400],
  },
  [`& .${linearProgressClasses.bar}`]: {
    borderRadius: '100px',
    backgroundColor: theme.palette.primary.main,
  },
}));

const UNSCRIPTED_SKILLS = {
  fluency: 'Fluency',
  grammar: 'Grammar',
  vocabulary: 'Vocabulary',
  pronunciation: 'Pronunciation',
};

const SKILLS = {
  fluency: 'Reading Fluency',
  pronunciation: 'Pronunciation',
};

const MAX_SCORE = {
  ielts: 9,
  pte: 90,
  cefr: 6,
};

type ReportScoreProps = {
  overall: number;
  skillResults: SkillResult;
  test: string;
  unscripted: boolean;
};

export default function ReportChart({ overall, skillResults, test, unscripted }: ReportScoreProps) {
  const progress = useMemo(
    () =>
      skillResults &&
      Object.fromEntries([
        ['overall', (overall / MAX_SCORE[test as keyof typeof MAX_SCORE]) * 100],
        ...Object.entries(skillResults).map(([skill, score]) => [
          skill,
          test === 'ielts'
            ? scaleIeltsScore((score / MAX_SCORE[test as keyof typeof MAX_SCORE]) * 100)
            : (score / MAX_SCORE[test as keyof typeof MAX_SCORE]) * 100,
        ]),
        // eslint-disable-next-line react-hooks/exhaustive-deps
      ]),
    [overall, skillResults, test]
  );
  let overallScore: number | string;
  if (test === 'cefr') {
    if (typeof overall === 'number') {
      overallScore = CEFR_BANDS[overall - 1];
    } else {
      overallScore = overall;
    }
  } else if (test === 'ielts') {
    overallScore = scaleIeltsScore(overall);
  } else {
    overallScore = overall;
  }

  return (
    <div>
      <Stack sx={{ width: '100%' }} direction="row" gap={6} flexWrap="wrap">
        <Stack flexBasis="30%" flexGrow={1} gap={3.5} flexWrap="wrap">
          <BoldText>Overall Score</BoldText>
          <OverallContainer>
            <CircularProgress
              variant="determinate"
              sx={{ color: (theme) => theme.palette.grey[400] }}
              size="clamp(40px, 2.4em, 200px)"
              thickness={4}
              value={100}
            />
            <InnerCircularProgress
              variant="determinate"
              size="clamp(40px, 2.4em, 200px)"
              thickness={4}
              value={progress ? progress.overall : 0}
            />
            <ScoreStyle>{overallScore}</ScoreStyle>
          </OverallContainer>
        </Stack>
        <Stack flexBasis="60%" flexGrow={1} gap={'clamp(1em, 2.5vw, 1.5em)'}>
          <BoldText>Skills</BoldText>
          {Object.entries(unscripted ? UNSCRIPTED_SKILLS : SKILLS).map(([skill, title]) =>
            skillResults[skill as keyof SkillResult] === undefined ? null : (
              <div key={`skill-${skill}`}>
                <Stack direction="row" justifyContent="space-between" mr={1} mb={0.8}>
                  <TextStyle>{title}</TextStyle>
                </Stack>
                <LinearProgressContainer>
                  <ScoreMaxMinText>{test === 'cefr' ? CEFR_BANDS[0] : 0}</ScoreMaxMinText>
                  <LinearProgressBarContainer>
                    <ScoreIndicatorText
                      progressPosition={
                        (skillResults[skill as keyof SkillResult] || 0.0) /
                        MAX_SCORE[test as keyof typeof MAX_SCORE]
                      }
                    >
                      {test === 'cefr'
                        ? CEFR_BANDS[
                            ((skillResults[skill as keyof SkillResult] || 1) -
                              1) as keyof typeof CEFR_BANDS
                          ]
                        : typeof skillResults[skill as keyof SkillResult] === 'number'
                          ? skillResults[skill as keyof SkillResult]?.toFixed(1)
                          : skillResults[skill as keyof SkillResult]}
                    </ScoreIndicatorText>
                    <BorderLinearProgress
                      variant="determinate"
                      value={progress ? progress[skill] : 0}
                    />
                  </LinearProgressBarContainer>
                  <ScoreMaxMinText>
                    {test === 'cefr'
                      ? CEFR_BANDS[MAX_SCORE[test] - 1]
                      : MAX_SCORE[test as keyof typeof MAX_SCORE]}
                  </ScoreMaxMinText>
                </LinearProgressContainer>
              </div>
            )
          )}
        </Stack>
      </Stack>
    </div>
  );
}
