import { v4 as uuidv4 } from 'uuid';
//mui
import { Typography, Box, Tab, Tabs } from '@mui/material';
import { styled } from '@mui/material/styles';
//type
import { LcatResult } from '../@types/lcat';
// component
import ReportChart from './ReportChart';
import { useMemo, useState } from 'react';
import FeedbackTabsSection from './FeedbackTabsSection';
import FeedbackSection from './FeedbackSection';
import PronunciationResult from './PronunciationResult';
// utils
import parseFeedback from '../utils/parseFeedback';

const ScoreContainer = styled('div')(({ theme }) => ({
  width: 'fit-content',
  margin: '18px auto',
  backgroundColor: theme.palette.common.white,
  boxShadow: `0 0 0 20px ${theme.palette.common.white}`,
  position: 'relative',
  bottom: '-40px',
}));

const AverageScoreStyle = styled(Typography)(() => ({
  fontWeight: 500,
  fontSize: '16px',
  lineHeight: '22px',
  display: 'flex',
  alignItems: 'center',
  padding: '8px 20px',
  borderRadius: '200px',
  backgroundColor: 'rgba(132, 186, 116, 0.2)',
  color: '#599F44',
}));

const ReportContainer = styled(Box)(() => ({
  borderBlock: '1px dashed #599F44',
  paddingBlock: '60px 3rem',
  display: 'flex',
  flexDirection: 'column',
  gap: '3rem',
  width: '100%',
}));

const TabContainer = styled(Box)(() => ({
  alignSelf: 'center',
  width: 'clamp(280px, 70%, 500px)',
  borderRadius: '100px',
  backgroundColor: '#E6E8EC',
  display: 'flex',
  '& .MuiTabs-root': {
    width: '100%',
  },
}));

const TabButton = styled(Tab)(({ theme }) => ({
  flex: '1 1 100%',
  fontSize: '0.9rem',
  borderRadius: '100px',
  color: '#777E90',
  '&.Mui-selected': {
    backgroundColor: theme.palette.primary.main,
    color: '#FFFFFF !important',
  },
}));

const FEEDBACK_TABS = [
  {
    title: 'Content relevance',
    key: 'content',
  },
  {
    title: 'Pronunciation',
    key: 'pronunciation',
  },
  {
    title: 'Fluency',
    key: 'fluency',
  },
  {
    title: 'Reading',
    key: 'reading',
  },
];

type ResultSectionProps = {
  result: LcatResult;
  speechPhrase?: string;
};

function a11yProps(index: number) {
  return {
    id: `scrollable-auto-tab-${index}`,
    'aria-controls': `scrollable-auto-tabpanel-${index}`,
  };
}

function ResultSection(props: ResultSectionProps) {
  const [tab, setTab] = useState(0);
  const [test, setTest] = useState('ielts');

  const { result, speechPhrase } = props;
  const skillResult = {
    fluency: result.fluency.english_proficiency_scores[`mock_${test}`].prediction,
    pronunciation: result.pronunciation.english_proficiency_scores[`mock_${test}`].prediction,
  };

  const { metaData, fluencyFeedback, readingFeedback } = parseFeedback(result, speechPhrase);

  function handleChange(_event: any, newValue: number) {
    setTab(newValue);
    switch (newValue) {
      case 0:
        setTest('ielts');
        break;
      case 1:
        setTest('pte');
        break;
      case 2:
        setTest('cefr');
        break;
      default:
        break;
    }
  }

  const feedbackSections = useMemo(() => {
    const id = uuidv4();
    return {
      content: <FeedbackSection uniqueId={`${id}-content`} data={metaData} />,
      pronunciation: (
        <PronunciationResult uniqueId={`${id}-pronunciation`} words={result.pronunciation.words} />
      ),
      fluency: <FeedbackSection uniqueId={`${id}-fluency`} data={fluencyFeedback} />,
      reading: <FeedbackSection uniqueId={`${id}-reading`} data={readingFeedback} />,
    };
  }, [result, metaData, fluencyFeedback, readingFeedback]);

  return (
    <Box style={{ width: '100%' }}>
      <ScoreContainer>
        <AverageScoreStyle>Speaking Report</AverageScoreStyle>
      </ScoreContainer>
      <ReportContainer>
        <TabContainer>
          <Tabs
            value={tab}
            onChange={handleChange}
            aria-label="tabs"
            TabIndicatorProps={{
              style: { display: 'none' },
            }}
          >
            <TabButton label="Mock IELTS" {...a11yProps(0)} />
            <TabButton label="PTE" {...a11yProps(1)} />
            <TabButton label="CEFR" {...a11yProps(2)} />
          </Tabs>
        </TabContainer>
        <ReportChart
          overall={
            result.overall.english_proficiency_scores
              ? result.overall.english_proficiency_scores[`mock_${test}`].prediction
              : 0
          }
          skillResults={skillResult}
          test={test}
          unscripted={false}
        />
        <FeedbackTabsSection feedbackTabs={FEEDBACK_TABS} feedbackSections={feedbackSections} />
      </ReportContainer>
    </Box>
  );
}

export default ResultSection;
