import axios, { AxiosRequestConfig } from 'axios';
import { NextApiRequest, NextApiResponse } from 'next';

/**
 * IELTS scoring handler
 *
 * @param {NextApiRequest} req - Next.js request object
 * @param {NextApiResponse} res - Next.js response object
 */
const handleIelts = async (req: NextApiRequest, res: NextApiResponse) => {
  const { method } = req;

  switch (method) {
    /**
     * Get the result from IELTS scoring service
     *
     * @body {string} answer: audio transcript
     * @body {string} base64Audio: current audio file in base64 string format
     * @body {number} duration: audio duration
     *
     * @response {number} lexicalResources
     * @response {number} grammar
     * @response {number} fluencyAndCoherence
     * @response {number} pronunciation
     * @response {number} total
     */
    case 'POST':
      const { dialogue, ...lcPayload }: { dialogue: string[] } = req.body;

      try {
        const response = await axios.request({
          method: 'POST',
          url: `${process.env.IELTS_SERVICE_URL}`,
          data: lcPayload,
          headers: {
            'api-key': `${process.env.API_KEY_IELTS}`,
            'lc-beta-features': 'true',
          },
        } as AxiosRequestConfig);

        if (
          !dialogue ||
          dialogue.length === 0 ||
          response.data?.metadata.content_relevance === 'NOT_RELEVANT'
        )
          return res.status(200).json(response.data);

        const gptResponse = await axios.request({
          method: 'POST',
          url: process.env.CHATGPT_API_URL || '',
          data: {
            model: 'gpt-3.5-turbo',
            max_tokens: 50,
            messages: [
              {
                role: 'system',
                content:
                  'You are an examiner who is conducting a short conversation speaking test. You need to ask exactly 5 questions and then end the conversation.',
              },
              ...dialogue.map((message, idx) => ({
                role: idx % 2 === 0 ? 'assistant' : 'user',
                content: message,
              })),
              {
                role: 'user',
                content: response.data.metadata.predicted_text,
              },
            ],
          },
          headers: {
            Authorization: `Bearer ${process.env.CHATGPT_API_KEY}`,
          },
        } as AxiosRequestConfig);

        res.status(200).json({
          ...response.data,
          nextQuestion: gptResponse.data.choices[0].message.content,
        });
      } catch (error) {
        res.status(500).json({ message: 'Something went wrong' });
      }
      break;

    default:
      res.setHeader('Allow', ['POST']);
      res.status(405).end(`Method ${method} Not Allowed`);
      break;
  }
};

export default handleIelts;

export const config = {
  api: {
    bodyParser: {
      sizeLimit: '100mb',
    },
  },
};
