import axios, { AxiosRequestConfig } from 'axios';
import { NextApiRequest, NextApiResponse } from 'next';

/**
 * Writing grading handler
 *
 * @param {NextApiRequest} req - Next.js request object
 * @param {NextApiResponse} res - Next.js response object
 */
const handler = async (req: NextApiRequest, res: NextApiResponse) => {
  const { method } = req;

  switch (method) {
    /**
     * Get the result from LCAT writing API
     *
     * @body {string} text: User's answer
     * @body {string} context.question: The question the user should be answering
     * @body {string} context.context_description: A short description of the context the user is expected to answer within
     * @body {string} context.valid_answer_description: A short description of the valid answer the user is expected to give
     *
     * @response {object}: response format from LCAT API.
     */
    case 'POST':
      await axios
        .request({
          method: 'POST',
          url: `${process.env.WRITING_API_URL}`,
          headers: {
            'Content-Type': 'application/json',
            'api-key': process.env.WRITING_API_KEY,
            'lc-beta-features': 'true',
          },
          data: req.body,
        } as AxiosRequestConfig)
        .then((response) => {
          if (process.env.FLAG_CONVERT_WRITING_IELTS) {
            response.data.overall.english_proficiency_scores.mock_ielts.prediction *= 9 / 100;
          }
          res.status(200).json({
            ...response.data,
          });
        })
        .catch((error) => {
          const errorMsg = error?.response?.data?.detail?.[0]?.msg;
          res.status(500).json({ message: errorMsg || 'Something went wrong' });
        });
      break;

    default:
      res.setHeader('Allow', ['POST']);
      res.status(405).end(`Method ${method} Not Allowed`);
      break;
  }
};

export default handler;
