import { useRouter } from 'next/router';
import { useState, useEffect, ChangeEvent } from 'react';
import { useLocalStorage } from '../../../hooks/useLocalStorage';
import axios from 'axios';
// @mui
import { styled, alpha } from '@mui/material/styles';
import {
  Typography,
  Card,
  Stack,
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Backdrop,
  CircularProgress,
  Box,
} from '@mui/material';
// components
import Page from '../../../components/Page';
import Layout from '../../../layouts';
import Breadcrumb from '../../../components/Breadcrumb';
import TestSection from '../../../layouts/TestSection';
import ResultSection from '../../../layouts/ResultSection';
import FinalReport from '../../../layouts/FinalReport';
// types
import { GetStaticPaths, GetStaticProps } from 'next';
import { Products } from '../../../@types/products';
import { LcatResult, TestResult } from '../../../@types/lcat';
// config
import products from '../../../utils/products.json';
import { SampleResult } from 'src/@types/final-report';
import ApiPricingCard from 'src/components/ApiPricingCard';

const RootStyle = styled('div')(({ theme }) => ({
  minHeight: 'calc(100vh - 130px)',
  backgroundColor: alpha(theme.palette.primary.light, 0.1),
  paddingBlock: '45px 60px',
}));

const containerStyle = {
  width: 'clamp(max(300px, min(600px, 100%)), 90%, 1200px)',
  marginInline: 'auto',
};

const CardStyle = styled(Card)(() => ({
  display: 'flex',
  flexDirection: 'column',
  justifyContent: 'center',
  alignItems: 'center',
  padding: '20px 45px 35px 45px',
  borderRadius: '20px',
  ...containerStyle,
}));

const GreenTextStyle = styled(Typography)(({ theme }) => ({
  fontSize: '16px',
  fontWeight: '700',
  lineHeight: '48px',
  color: theme.palette.primary.main,
  alignSelf: 'flex-start',
}));

const ButtonStyle = styled(Button)(({ theme }) => ({
  padding: '16px 100px',
  fontSize: '16px',
  fontWeight: '700',
  lineHeight: '22px',
  borderRadius: '4px',
  marginTop: '32px',
  alignSelf: 'flex-end',
}));

const ButtonContainedStyle = styled(ButtonStyle)(({ theme }) => ({
  color: theme.palette.common.white,
  backgroundColor: theme.palette.primary.main,
  '&:hover': {
    backgroundColor: theme.palette.primary.main,
  },
}));

const ButtonOutlinedStyle = styled(ButtonStyle)(({ theme }) => ({
  border: `1px solid ${theme.palette.primary.main}`,
  '&:hover': {
    backgroundColor: 'unset',
  },
}));

Accent.getLayout = function getLayout(page: React.ReactElement) {
  return <Layout>{page}</Layout>;
};

export default function Accent({ products }: { products: Products }) {
  const { query } = useRouter();
  const router = useRouter();
  const { level, accent } = query;
  const [customText, setCustomText] = useState('');
  const [testNumber, setTestNumber] = useLocalStorage(`scripted-${level}-test`, 0);
  const levelData = products.scripted.levels[level as string];
  const [result, setResult] = useLocalStorage(`result-${level}`, '');
  const [ielstScore, setIELTS] = useLocalStorage(`ielts-${level}`, '');
  const [pteScore, setPTE] = useLocalStorage(`pte-${level}`, '');
  // const [phonemes, setPhonemes] = useLocalStorage(`phonemes-${level}`, '');

  const [blobFile, setBlobFile] = useState<any>();
  const [returnedData, setData] = useState<LcatResult | null>(null);
  const [openErrorDialog, setOpenErrorDialog] = useState(false);
  const [errorContent, setErrorContent] = useState('');
  const [submitLoading, setSubmitLoading] = useState(false);

  if (typeof window !== 'undefined')
    window.localStorage.setItem(`${level}-accent`, JSON.stringify(accent));

  useEffect(() => {
    const fetchResult = async () => {
      setSubmitLoading(true);
      const reader = new window.FileReader();
      reader.readAsDataURL(blobFile.blob);
      reader.onload = async () => {
        if (typeof reader.result === 'string') {
          const base64ConvertedFile = reader.result.split(',')[1];
          await axios
            .post('/api/grade/', {
              base64_audio: base64ConvertedFile,
              audio_format: 'mp3',
              text: levelData.data[testNumber as number].text || customText,
              accent: accent,
            })
            .then((response) => {
              setData(response.data);
              if (level !== 'custom') saveResult(response.data);
            })
            .catch((error) => {
              setErrorContent(error.message);
              setOpenErrorDialog(true);
            })
            .finally(() => {
              setSubmitLoading(false);
            });
        }
      };
    };

    if (blobFile) fetchResult();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [blobFile, levelData, testNumber]);

  const saveResult = (responseData: LcatResult) => {
    let resultData = [] as SampleResult[];
    let ielts = [] as TestResult[];
    let pte = [] as TestResult[];
    // let phonemesData = {} as Record<string, PhoneResult>;

    if (result !== '') {
      resultData = JSON.parse(result);
    }

    if (ielstScore != '') {
      ielts = JSON.parse(ielstScore);
    }

    if (pteScore != '') {
      pte = JSON.parse(pteScore);
    }

    // if (phonemes !== '') {
    //   phonemesData = JSON.parse(phonemes) as Record<string, PhoneResult>;
    // }

    const recentQuestion = levelData.data[testNumber as number].text || customText;
    resultData = resultData.filter((question) => question.text !== recentQuestion);
    ielts = ielts.filter((question) => question.text !== recentQuestion);
    pte = pte.filter((question) => question.text !== recentQuestion);
    const score = responseData.pronunciation.overall_score;

    if (score) {
      resultData.push({ text: recentQuestion, score: score });
    } else {
      resultData.push({ text: recentQuestion, score: 0 });
    }
    setResult(JSON.stringify(resultData));

    ielts.push({
      text: recentQuestion,
      fluency: responseData.fluency.english_proficiency_scores.mock_ielts.prediction,
      pronunciation: responseData.pronunciation.english_proficiency_scores.mock_ielts.prediction,
      overall: responseData.overall.english_proficiency_scores.mock_ielts.prediction,
    });
    setIELTS(JSON.stringify(ielts));

    pte.push({
      text: recentQuestion,
      fluency: responseData.fluency.english_proficiency_scores.mock_pte.prediction,
      pronunciation: responseData.pronunciation.english_proficiency_scores.mock_pte.prediction,
      overall: responseData.overall.english_proficiency_scores.mock_pte.prediction,
    });
    setIELTS(JSON.stringify(ielts));
    setPTE(JSON.stringify(pte));

    // const recentWordsList = responseData.pronunciation.words;
    // if (recentWordsList) {
    //   recentWordsList.forEach((word) => {
    //     word.phonemes.forEach((phone) => {
    //       const label = phone.ipa_label;
    //       if (phonemesData[label]) {
    //         const recentTotalScore = phonemesData[label].total_score + phone.phoneme_score;
    //         const frequency = phonemesData[label].occurence + 1;
    //         phonemesData[label] = {
    //           word: word.word_text,
    //           total_score: recentTotalScore,
    //           occurence: frequency,
    //         };
    //       } else {
    //         phonemesData[label] = {
    //           word: word.word_text,
    //           total_score: phone.phoneme_score,
    //           occurence: 1,
    //         };
    //       }
    //     });
    //   });
    // }
    // setPhonemes(JSON.stringify(phonemesData));
  };

  const handleSubmit = (blobFile: any) => {
    setBlobFile(blobFile);
  };

  const clearData = () => {
    setBlobFile(null);
    setData(null);
    setCustomText('');
  };

  const handleNextQuestion: any = () => {
    clearData();

    if (testNumber === levelData.data.length) {
      window.localStorage.clear();
      switch (level) {
        case 'beginner':
          router.push('/scripted/intermediate/');
          break;
        case 'intermediate':
          router.push('/scripted/advanced/');
          break;
        case 'advanced':
          router.push('/');
          break;
      }
    } else {
      setTestNumber(testNumber + 1);
    }
  };

  const handleReturnHomepage = () => {
    clearData();
    router.push('/');
  };

  const handleClose = () => {
    setOpenErrorDialog(false);
  };

  const handleCustomTextChange = (event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
    setCustomText(event.target.value);
  };

  const nameButton = () => {
    if (testNumber === levelData.data.length - 1) {
      return 'View Report';
    } else {
      return 'Next Question';
    }
  };

  return (
    <Page title={`${level}`}>
      {Breadcrumb([
        { title: 'External Demo', location: '/', disabled: false },
        { title: 'Scripted Speech Assessment', location: '/scripted', disabled: false },
        {
          title: `${levelData.title} (${(accent as string).toUpperCase()} accent)`,
          location: `/scripted/${level}`,
          disabled: true,
        },
      ])}
      {testNumber !== levelData.data.length || level === 'custom' ? (
        <RootStyle>
          <CardStyle raised={false}>
            {level !== 'custom' && (
              <GreenTextStyle>
                {`Sample: ${(testNumber as number) + 1}/${levelData.data.length}`}
              </GreenTextStyle>
            )}
            <TestSection
              text={levelData.data[testNumber as number].text}
              audioUrl={levelData.data[testNumber as number].audioUrl}
              blobUrl={blobFile?.blobURL || ''}
              customText={customText}
              handleCustomTextChange={handleCustomTextChange}
              handleSubmit={handleSubmit}
            />
            {returnedData && (
              <>
                <ResultSection
                  result={returnedData}
                  speechPhrase={levelData.data[testNumber as number].text || customText}
                />
                {level === 'custom' ? (
                  <Stack direction="row" gap={2} sx={{ alignSelf: 'flex-end' }}>
                    <ButtonOutlinedStyle variant="outlined" disableElevation onClick={clearData}>
                      Clear
                    </ButtonOutlinedStyle>
                    <ButtonContainedStyle
                      variant="contained"
                      disableElevation
                      onClick={handleReturnHomepage}
                      sx={{ paddingInline: '60px' }}
                    >
                      Return to home page
                    </ButtonContainedStyle>
                  </Stack>
                ) : (
                  <ButtonContainedStyle
                    variant="contained"
                    disableElevation
                    onClick={handleNextQuestion}
                  >
                    {nameButton()}
                  </ButtonContainedStyle>
                )}
              </>
            )}
          </CardStyle>

          <Box sx={containerStyle} mt={5}>
            <ApiPricingCard />
          </Box>

          <Dialog open={openErrorDialog} onClose={handleClose}>
            <DialogTitle color="error">Error</DialogTitle>
            <DialogContent>
              <DialogContentText>{errorContent}</DialogContentText>
            </DialogContent>
            <DialogActions>
              <Button color="error" onClick={handleClose}>
                Close
              </Button>
            </DialogActions>
          </Dialog>

          <Backdrop sx={{ zIndex: '99999' }} open={submitLoading}>
            <CircularProgress color="primary" size={100} />
          </Backdrop>
        </RootStyle>
      ) : (
        <FinalReport
          handleNextQuestion={handleNextQuestion}
          level={level}
          sampleResult={JSON.parse(result)}
          ieltsResult={JSON.parse(ielstScore)}
          pteResult={JSON.parse(pteScore)}
          // phonesResult={JSON.parse(phonemes)}
        />
      )}
    </Page>
  );
}

export const getStaticPaths: GetStaticPaths = async () => ({
  paths: [
    ...Object.keys(products.scripted.levels).map((level) => ({
      params: { level, accent: 'us' },
    })),
    ...Object.keys(products.scripted.levels).map((level) => ({
      params: { level, accent: 'uk' },
    })),
  ],
  fallback: false,
});

export const getStaticProps: GetStaticProps<{
  products: Products;
}> = async () => ({
  props: {
    products,
  },
});
