import { useRouter } from 'next/router';
import { useState, useEffect, ChangeEvent } from 'react';
import { useLocalStorage } from '../../hooks/useLocalStorage';
import axios from 'axios';
// @mui
import { styled, alpha } from '@mui/material/styles';
import { Typography, Card, Button, Stack, Box } from '@mui/material';
// components
import Page from '../../components/Page';
import Layout from '../../layouts';
import Breadcrumb from '../../components/Breadcrumb';
import IeltsTestSection from '../../layouts/IeltsTestSection';
import IeltsResultSection from '../../layouts/IeltsResultSection';
import Dialogue from '../../layouts/Dialogue';
import IeltsResultTabs from '../../layouts/IeltsResultTabs';
import LoadingBackdrop from 'src/components/LoadingBackdrop';
import ErrorDialog from 'src/components/ErrorDialog';
// types
import { GetStaticPaths, GetStaticProps } from 'next';
import { Products } from '../../@types/products';
import { IeltsResult } from '../../@types/ielts';
// config
import products from '../../utils/products.json';
import { TESTS, SKILLS, convertCefrToNumber, convertNumberToCefr } from '../../utils/scoreUtils';
import ApiPricingCard from 'src/components/ApiPricingCard';

export const RootStyle = styled('div')(({ theme }) => ({
  minHeight: 'calc(100vh - 130px)',
  backgroundColor: alpha(theme.palette.primary.light, 0.1),
  paddingBlock: '45px 60px',
}));

const containerStyle = {
  width: 'clamp(max(300px, min(600px, 100%)), 90%, 1200px)',
  marginInline: 'auto',
};

export const CardStyle = styled(Card)(() => ({
  display: 'flex',
  flexDirection: 'column',
  justifyContent: 'center',
  alignItems: 'center',
  padding: '25px 45px 35px 45px',
  borderRadius: '20px',
  ...containerStyle,
}));

const GreenTextStyle = styled(Typography)(({ theme }) => ({
  fontSize: '16px',
  fontWeight: '700',
  lineHeight: '24px',
  color: theme.palette.primary.main,
  alignSelf: 'flex-start',
  minWidth: '110px',
}));

const QuestionText = styled(Typography)(() => ({
  fontSize: '16px',
  fontWeight: '500',
  lineHeight: '24px',
  minWidth: '200px',
  flex: '1',
}));

const ButtonStyle = styled(Button)(({ theme }) => ({
  padding: '16px 100px',
  fontSize: '16px',
  fontWeight: '700',
  lineHeight: '22px',
  borderRadius: '4px',
  marginTop: '32px',
  alignSelf: 'flex-end',
  color: theme.palette.common.white,
  backgroundColor: theme.palette.primary.main,
  '&:hover': {
    backgroundColor: theme.palette.primary.main,
  },
}));

Ielts.getLayout = function getLayout(page: React.ReactElement) {
  return <Layout>{page}</Layout>;
};

export default function Ielts({ products }: { products: Products }) {
  const router = useRouter();
  const { level } = router.query;
  const levelData = products.unscripted.levels[level as string];
  const videoUrl = levelData.data.video || '';
  const [testNumber, setTestNumber] = useLocalStorage(`unscripted-${level}-test`, 0);
  const [reviewTestNumber, setReviewTestNumber] = useState(-1);
  const questions = levelData.data.questions;
  const pictureData = levelData.data.pictures ? levelData.data.pictures[testNumber] : '';
  const videoContext = levelData.data.context ? levelData.data.context[testNumber] : '';
  const useCaseTitle = levelData.title;
  const minDuration = levelData.minDuration ?? 10;
  const maxDuration = levelData.maxDuration ?? 119;
  const [customText, setCustomText] = useState('');
  const [dialogue, setDialogue] = useLocalStorage<string[]>(
    'unscripted-dialogue',
    level === 'dialogue' ? [questions[0]] : []
  );

  const [blobFile, setBlobFile] = useState<any>();
  const [duration, setDuration] = useState<number>(0);
  const [returnedData, setData] = useState<IeltsResult | null>(null);
  const [results, setResults] = useLocalStorage<IeltsResult[]>(`results-unscripted-dialogue`, []);
  const [openErrorDialog, setOpenErrorDialog] = useState(false);
  const [errorContent, setErrorContent] = useState('');
  const [submitLoading, setSubmitLoading] = useState(false);

  const getAverageResult = (_results: IeltsResult[]) => {
    const results = JSON.parse(JSON.stringify(_results)) as IeltsResult[];
    const result = results.reduce((prev, result) => {
      TESTS.forEach((test) => {
        SKILLS.forEach((skill) => {
          if (test === 'cefr') {
            prev[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction =
              convertCefrToNumber(
                prev[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                  .prediction
              );
            result[skill as keyof IeltsResult].english_proficiency_scores[
              `mock_${test}`
            ].prediction = convertCefrToNumber(
              result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                .prediction
            );
          }

          if (test === 'pte') {
            prev[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction =
              parseFloat(
                prev[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                  .prediction
              );
            result[skill as keyof IeltsResult].english_proficiency_scores[
              `mock_${test}`
            ].prediction = parseFloat(
              result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                .prediction
            );
          }

          prev[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction +=
            result[skill as keyof IeltsResult].english_proficiency_scores[
              `mock_${test}`
            ].prediction;
        });

        if (test === 'cefr') {
          prev.overall.english_proficiency_scores[`mock_${test}`].prediction = convertCefrToNumber(
            prev.overall.english_proficiency_scores[`mock_${test}`].prediction
          );
          result.overall.english_proficiency_scores[`mock_${test}`].prediction =
            convertCefrToNumber(
              result.overall.english_proficiency_scores[`mock_${test}`].prediction
            );
        }

        if (test === 'pte') {
          prev.overall.english_proficiency_scores[`mock_${test}`].prediction = parseFloat(
            prev.overall.english_proficiency_scores[`mock_${test}`].prediction
          );
          result.overall.english_proficiency_scores[`mock_${test}`].prediction = parseFloat(
            result.overall.english_proficiency_scores[`mock_${test}`].prediction
          );
        }

        prev.overall.english_proficiency_scores[`mock_${test}`].prediction +=
          result.overall.english_proficiency_scores[`mock_${test}`].prediction;
      });
      return prev;
    });

    TESTS.forEach((test) => {
      result.overall.english_proficiency_scores[`mock_${test}`].prediction /= results.length;
      if (test !== 'ielts')
        result.overall.english_proficiency_scores[`mock_${test}`].prediction = Math.round(
          result.overall.english_proficiency_scores[`mock_${test}`].prediction
        );
      if (test === 'cefr')
        result.overall.english_proficiency_scores[`mock_${test}`].prediction = convertNumberToCefr(
          result.overall.english_proficiency_scores[`mock_${test}`].prediction
        );
      SKILLS.forEach((skill) => {
        result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction /=
          results.length;
        if (test !== 'ielts')
          result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction =
            Math.round(
              result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                .prediction
            );
        if (test === 'cefr')
          result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`].prediction =
            convertNumberToCefr(
              result[skill as keyof IeltsResult].english_proficiency_scores[`mock_${test}`]
                .prediction
            );
      });
    });
    return result;
  };

  useEffect(() => {
    if (testNumber >= questions.length) {
      setTestNumber(0);
      setData(null);
      setResults([]);
    }

    if (level !== 'dialogue') return;

    if (testNumber === 0) {
      setResults([]);
      setDialogue(level === 'dialogue' ? [questions[0]] : []);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  useEffect(() => {
    if (reviewTestNumber === -1) setReviewTestNumber(testNumber);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [testNumber]);

  useEffect(() => {
    const fetchResult = async () => {
      setSubmitLoading(true);
      const reader = new window.FileReader();
      reader.readAsDataURL(blobFile.blob);
      reader.onload = async () => {
        if (typeof reader.result === 'string') {
          const base64ConvertedFile = reader.result.split(',')[1];
          const contextDescription =
            (pictureData && pictureData.description) || (videoUrl && videoContext) || '';
          await axios
            .post('/api/unscripted-scoring/', {
              audio_base64: base64ConvertedFile,
              audio_format: 'mp3',
              context: {
                question:
                  (level === 'dialogue' && dialogue.length > 0 && dialogue[dialogue.length - 1]) ||
                  questions[testNumber] ||
                  customText,
                ...(contextDescription && { context_description: contextDescription }),
              },
              ...(level === 'dialogue' && { dialogue }),
            })
            .then((response) => {
              const objectResult = response.data;
              // let data = {} as any;

              // for (const key in objectResult) {
              //   if (Object.hasOwnProperty.call(objectResult, key)) {
              //     data = {
              //       ...data,
              //       [`${key}`]:
              //         objectResult[key]['english_proficiency_scores']['mock_ielts']['prediction'],
              //     };
              //   }
              // }

              // return data;
              return objectResult;
            })
            .then((obj: IeltsResult) => {
              if (level === 'dialogue') {
                if (obj.metadata.content_relevance === 'NOT_RELEVANT')
                  throw new Error('Your answer is not relevant to the question. Please try again.');

                setDialogue((dialogue) => {
                  dialogue[testNumber * 2 + 1] = obj.metadata.predicted_text;
                  if (obj.nextQuestion) dialogue[(testNumber + 1) * 2] = obj.nextQuestion;
                  return dialogue;
                });
              }

              setData(obj);
            })
            .catch((error) => {
              setErrorContent(error.message);
              setOpenErrorDialog(true);
            })
            .finally(() => {
              setSubmitLoading(false);
            });
        }
      };
    };

    if (blobFile && duration) fetchResult();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [blobFile, duration, questions, testNumber]);

  const clearUserData = () => {
    setBlobFile(null);
    setDuration(0);
  };

  const handleSubmitRecording = (blobFile: any) => {
    setBlobFile(blobFile);
  };

  const handleNextQuestion: any = () => {
    if (level === 'dialogue') {
      setResults((results) => {
        results.push(returnedData as IeltsResult);
        return results;
      });
    }

    clearUserData();
    setData(null);
    setCustomText('');

    if (testNumber === questions.length - 1) {
      if (level === 'dialogue') {
        setDialogue([]);
        setTestNumber(testNumber + 1);
      } else {
        setTestNumber(0);
        router.push('/');
      }
    } else {
      setTestNumber(testNumber + 1);
      setReviewTestNumber(testNumber + 1);
    }
  };

  const handleBackToHome = () => {
    setResults([]);
    setTestNumber(0);
    router.push('/');
  };

  const handleReviewResult = (question: number) => {
    setReviewTestNumber(question);
  };

  const handleClose = () => {
    setOpenErrorDialog(false);
  };

  const handleCustomTextChange = (event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
    setCustomText(event.target.value);
  };

  const nameButton = () => {
    if (testNumber === -1) {
      return 'Skip the video';
    } else if (testNumber === questions.length - 1) {
      if (level === 'dialogue') return 'View Report';
      return 'Return to home page';
    } else {
      return 'Next Question';
    }
  };

  return (
    <Page title={`Unscripted - ${useCaseTitle}`}>
      {Breadcrumb([
        { title: 'External Demo', location: '/', disabled: false },
        { title: 'Unscripted Speech Assessment', location: '/unscripted', disabled: false },
        { title: useCaseTitle, location: `/unscripted/${level}`, disabled: true },
      ])}
      <RootStyle>
        <CardStyle raised={false}>
          {level !== 'dialogue' || testNumber < questions.length ? (
            <>
              <Stack direction="row" gap={1} flexWrap="wrap" width="100%">
                {level !== 'custom' && level !== 'dialogue' && (
                  <GreenTextStyle>
                    {`Question: ${(testNumber as number) + 1}/${questions.length}`}
                  </GreenTextStyle>
                )}
                {level === 'dialogue' && (
                  <QuestionText sx={{ marginBlock: '1rem' }}>
                    Answer the questions below
                  </QuestionText>
                )}
                {pictureData && <QuestionText>{questions[testNumber as number]}</QuestionText>}
              </Stack>
              {level === 'dialogue' && (
                <Dialogue
                  messages={dialogue.slice(0, (testNumber + 1) * 2 - (returnedData ? 0 : 1))}
                  selectedQuestion={reviewTestNumber}
                  sx={{ marginBottom: reviewTestNumber === testNumber ? '4rem' : '0' }}
                  onMessageClick={handleReviewResult}
                />
              )}
              {(level !== 'dialogue' || reviewTestNumber === testNumber) && (
                <IeltsTestSection
                  text={level === 'dialogue' ? undefined : questions[testNumber as number]}
                  clearData={clearUserData}
                  handleSubmitRecording={handleSubmitRecording}
                  handleUpdateDuration={(duration: number) => setDuration(duration)}
                  blobUrl={blobFile?.blobURL || ''}
                  customText={customText}
                  handleCustomTextChange={handleCustomTextChange}
                  picture={pictureData ? pictureData?.url : ''}
                  videoUrl={videoUrl}
                  minDuration={minDuration}
                  maxDuration={maxDuration}
                />
              )}
              {(returnedData ||
                (reviewTestNumber !== -1 &&
                  reviewTestNumber !== testNumber &&
                  results[reviewTestNumber])) && (
                <>
                  <IeltsResultSection
                    result={
                      reviewTestNumber !== testNumber
                        ? results[reviewTestNumber]
                        : (returnedData as unknown as IeltsResult)
                    }
                  />
                  {(level !== 'dialogue' || reviewTestNumber === testNumber) && (
                    <ButtonStyle variant="contained" disableElevation onClick={handleNextQuestion}>
                      {nameButton()}
                    </ButtonStyle>
                  )}
                </>
              )}
            </>
          ) : (
            results &&
            results.length !== 0 && (
              <Stack width="100%">
                <Typography fontWeight="700" fontSize="32px" alignSelf="center" mb={2}>
                  Average Score
                </Typography>
                <IeltsResultTabs result={getAverageResult(results) as IeltsResult} />
                <ButtonStyle variant="contained" disableElevation onClick={handleBackToHome}>
                  Back to home page
                </ButtonStyle>
              </Stack>
            )
          )}
        </CardStyle>

        <Box sx={containerStyle} mt={5}>
          <ApiPricingCard />
        </Box>

        <ErrorDialog onClose={handleClose} open={openErrorDialog} content={errorContent} />
        <LoadingBackdrop isLoading={submitLoading} />
      </RootStyle>
    </Page>
  );
}

export const getStaticPaths: GetStaticPaths = async () => ({
  paths: Object.keys(products.unscripted.levels).map((level) => ({
    params: { level },
  })),
  fallback: false,
});

export const getStaticProps: GetStaticProps<{
  products: Products;
}> = async () => ({
  props: {
    products,
  },
});
