import { useState } from 'react';
import axios from 'axios';
// @mui
import { styled } from '@mui/material/styles';
import { Typography, Stack, Button, TextField } from '@mui/material';
// components
import Page from '../components/Page';
import Layout from '../layouts';
import Breadcrumb from '../components/Breadcrumb';
import { RootStyle, CardStyle as Card } from './unscripted/[level]';
import LoadingBackdrop from 'src/components/LoadingBackdrop';
import ErrorDialog from 'src/components/ErrorDialog';
// types
import WritingResultSection from 'src/layouts/WritingResultSection';
import { LCWritingResult } from 'src/@types/writing';

const MAX_LENGTH = 1000;

const CardStyle = styled(Card)(() => ({
  alignItems: 'flex-start',
  gap: '2rem',
  padding: '35px 45px',
}));

Writing.getLayout = function getLayout(page: React.ReactElement) {
  return <Layout>{page}</Layout>;
};

export default function Writing() {
  const [description, setDescription] = useState('');
  const [text, setText] = useState('');
  const [returnedData, setData] = useState<LCWritingResult | null>(null);
  const [openErrorDialog, setOpenErrorDialog] = useState(false);
  const [errorContent, setErrorContent] = useState('');
  const [submitLoading, setSubmitLoading] = useState(false);

  const handleClose = () => {
    setOpenErrorDialog(false);
  };

  const handleTextChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setText(event.target.value);
  };

  const handleDescriptionChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    setDescription(event.target.value);
  };

  const handleSubmit = async () => {
    try {
      if (!text) throw new Error('Please enter your answer.');
      if (text.length > MAX_LENGTH) {
        throw new Error(`The text is too long. Please keep it under ${MAX_LENGTH} characters.`);
      }
      setSubmitLoading(true);
      const response = await axios.post('/api/writing-scoring', {
        text,
        ...(description && { context: { context_description: description } }),
      });
      setData(response.data);
    } catch (error) {
      setErrorContent(error.message);
      setOpenErrorDialog(true);
    } finally {
      setSubmitLoading(false);
    }
  };

  return (
    <Page title={`Writing assessment`}>
      {Breadcrumb([
        { title: 'External Demo', location: '/', disabled: false },
        { title: 'Writing Assessment', location: '/writing', disabled: true },
      ])}

      <RootStyle>
        <CardStyle raised={false}>
          <Stack gap={3} width="100%" alignItems="flex-start">
            <Typography variant="h4" sx={{ fontWeight: 700, lineHeight: '1.75rem' }}>
              Writing Instructions (optional)
            </Typography>
            <TextField
              fullWidth
              placeholder="Describe what the user should write about...."
              value={description}
              onChange={handleDescriptionChange}
            />
            <Typography variant="h4" sx={{ fontWeight: 700, lineHeight: '1.75rem' }}>
              Answer
            </Typography>
            <Stack gap={1} width="100%">
              <TextField
                fullWidth
                multiline
                minRows={4}
                placeholder="Type Your Answer...."
                value={text}
                onChange={handleTextChange}
              />
              <Typography
                sx={{ color: text.length > MAX_LENGTH ? 'error.main' : 'text.secondary' }}
                variant="subtitle2"
                fontWeight="500"
              >
                {text.length}/{MAX_LENGTH} characters
              </Typography>
            </Stack>
            <Button
              onClick={handleSubmit}
              variant="contained"
              sx={{ alignSelf: 'flex-end', padding: '0.6rem 1.5rem' }}
            >
              Submit
            </Button>
          </Stack>

          {returnedData && <WritingResultSection result={returnedData} />}
        </CardStyle>

        <ErrorDialog onClose={handleClose} open={openErrorDialog} content={errorContent} />
        <LoadingBackdrop isLoading={submitLoading} />
      </RootStyle>
    </Page>
  );
}
