import { LcatResult } from '../@types/lcat';
import { IeltsResult } from '../@types/ielts';
import { Feedback } from '../@types/feedback';
import SpeechRateChart from 'src/components/SpeechRateChart';
import DetailedTranscript from 'src/components/DetailedTranscript';
import DetailedGrammar from 'src/components/DetailedGrammar';
import DetailedVocabTranscript from 'src/components/DetailedVocabTranscript';
import DetailedTranscriptLegendVocab from 'src/components/DetailedTranscriptLegendVocab';
import DetailedTranscriptLegendFluency from 'src/components/DetailedTranscriptLegendFluency';
import { LCWritingResult, WritingMetrics } from 'src/@types/writing';
import { formatTime } from './time';

export function writingResultToIelts(result: LCWritingResult): IeltsResult {
  const TITLE_MAPPING: Record<keyof WritingMetrics, string> = {
    grammar_errors: 'Grammar',
    spelling_errors: 'Spelling',
    punctuation_errors: 'Punctuation',
    missing_word_errors: 'Missing word',
    word_choice_errors: 'Word choice',
    word_repetition_errors: 'Word repetition',
  };

  return {
    overall: result.overall,
    grammar: {
      ...result.grammar,
      metrics: {
        ...result.grammar.metrics,
        grammar_errors: Object.entries(result.writing.metrics)
          .map(([key, errors]) =>
            errors.map((error) => {
              error.title = TITLE_MAPPING[key as keyof WritingMetrics];
              return error;
            })
          )
          .flat()
          .sort((a, b) => a.start_index - b.start_index),
      },
    },
    vocabulary: result.vocabulary,
    metadata: {
      predicted_text: result.text,
      content_relevance: result.relevance?.relevance ?? '',
      content_relevance_feedback: result.relevance?.feedback ?? '',
    },
  } as IeltsResult;
}

export default function parseFeedback(result: LcatResult | IeltsResult, speechPhrase?: string) {
  const metaData: Feedback[] = !('metadata' in result)
    ? []
    : [
        {
          title: 'Transcript - What Was Said',
          content: result.metadata.predicted_text,
          speechPhrase: speechPhrase,
        },
        ...(result.metadata?.content_relevance_feedback
          ? [
              {
                title: 'Content relevance score',
                content: result.metadata.content_relevance_feedback,
                value: result.metadata?.content_relevance
                  ? typeof result.metadata.content_relevance === 'string'
                    ? result.metadata?.content_relevance.replace('_', ' ')
                    : result.metadata?.content_relevance
                  : '',
              },
            ]
          : []),
      ];
  const fluencyFeedback: Feedback[] = !('fluency' in result)
    ? []
    : [
        {
          title: 'Detailed transcript',
          ...(result.fluency.feedback?.tagged_transcript?.length && {
            children: (
              <DetailedTranscript
                transcript={result.fluency.feedback?.tagged_transcript}
                speechPhrase={speechPhrase}
              />
            ),
            value: (
              <DetailedTranscriptLegendFluency
                transcript={result.fluency.feedback?.tagged_transcript}
                speechPhrase={speechPhrase}
              />
            ),
          }),
        },
        {
          title: 'Speech rate',
          content: result.fluency.feedback?.speech_rate?.feedback_text,
          value: result.fluency.metrics?.speech_rate,
          children: result.fluency.metrics?.speech_rate_over_time &&
            result.fluency.metrics.speech_rate_over_time.length && (
              <SpeechRateChart
                speechRateSerie={
                  result.fluency.metrics.speech_rate_over_time.length == 1
                    ? [
                        result.fluency.metrics.speech_rate_over_time[0],
                        result.fluency.metrics.speech_rate_over_time[0],
                      ]
                    : result.fluency.metrics.speech_rate_over_time
                }
              />
            ),
        },
        {
          title: 'Number of pauses',
          content: result.fluency.feedback?.pauses?.feedback_text,
          value: result.fluency.metrics?.pauses,
        },
        {
          title: 'Number of filler words',
          content: result.fluency.feedback?.filler_words?.feedback_text,
          value: result.fluency.metrics?.filler_words,
        },
        {
          title: 'Filler words per minute',
          content: result.fluency.feedback?.filler_words_per_min?.feedback_text,
          value: result.fluency.metrics?.filler_words_per_min,
        },
      ];
  const grammarFeedback: Feedback[] = !('grammar' in result)
    ? []
    : [
        {
          title: 'Number of grammar mistakes',
          children: (
            <DetailedGrammar
              transcript={result.metadata.predicted_text}
              feedback={result.grammar.metrics}
            />
          ),
          value: result.grammar.metrics?.grammar_errors?.length ?? 0,
        },
        {
          title: 'Grammar complexity',
          value: result.grammar.metrics?.grammatical_complexity.replace('_', ' '),
        },
      ];

  const deduplicatedIdioms: Array<{
    start_index: number;
    end_index: number;
  }> = [];
  if ('vocabulary' in result && result.vocabulary.metrics?.idiom_details) {
    result.vocabulary.metrics.idiom_details?.forEach((idiom) => {
      if (
        !deduplicatedIdioms.length ||
        idiom.start_index > deduplicatedIdioms[deduplicatedIdioms.length - 1]?.end_index
      )
        deduplicatedIdioms.push(idiom);
    });
    result.vocabulary.metrics.idiom_details = deduplicatedIdioms;
  }
  const vocabularyFeedback: Feedback[] = !('vocabulary' in result)
    ? []
    : [
        {
          title: 'Detailed transcript',
          ...(result.vocabulary.feedback?.tagged_transcript?.length && {
            children: (
              <DetailedVocabTranscript
                transcript={result.vocabulary.feedback?.tagged_transcript}
                feedback={result.vocabulary.metrics}
              />
            ),
            value: <DetailedTranscriptLegendVocab feedback={result.vocabulary.metrics} />,
          }),
        },
        {
          title: 'Vocabulary complexity',
          value: result.vocabulary.metrics?.vocabulary_complexity.replace('_', ' '),
        },
      ];

  const readingFeedback: Feedback[] = !('reading' in result)
    ? []
    : [
        {
          title: 'Total time',
          value: formatTime(result.reading.total_time),
          plain: true,
        },
        {
          title: 'Words read',
          value: result.reading.words_read,
        },
        {
          title: 'Unique words read',
          value: result.reading.unique_words_read,
        },
        {
          title: 'Speed (WCPM)',
          value: Math.round(result.reading.speed_wpm_correct),
        },
        {
          title: 'Completion',
          value: `${result.reading.completion * 100}%`,
          plain: true,
        },
        {
          title: 'Reading time',
          value: formatTime(result.reading.reading_time),
          plain: true,
        },
        {
          title: 'Non-reading time',
          value: formatTime(result.reading.non_reading_time),
          plain: true,
        },
        {
          title: 'Words assigned',
          value: result.reading.words_assigned,
        },
        {
          title: 'Unique words assigned',
          value: result.reading.unique_words_assigned,
        },
        {
          title: 'Correct words read',
          value: result.reading.correct_words_read,
        },
        {
          title: 'Words added',
          value: result.reading.words_added,
        },
        {
          title: 'Words missed',
          value: result.reading.words_missed,
        },
        {
          title: 'Words replaced',
          value: result.reading.words_replaced,
        },
        {
          title: 'Speed wpm',
          value: Math.round(result.reading.speed_wpm),
        },
        {
          title: 'Accuracy',
          value: `${result.reading.accuracy * 100}%`,
          plain: true,
        },
      ];

  return {
    metaData,
    fluencyFeedback,
    grammarFeedback,
    vocabularyFeedback,
    readingFeedback,
  };
}
